/* 
   mkpsxiso (Java Edition) - Converts a standard ISO image to .bin/.cue (PSX)
   This software is based on Conyers' mkpsxiso - http://www.conyers.demon.co.uk
   Author: Bruno Freitas (aka Sparrow) - 03/2004
   Contact: bootsector@ig.com.br
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.io.*;

/** This class was written based on Conyers' mkpsxiso program
 * (http://www.conyers.demon.co.uk) which "translates" a plain ISO file into a
 * .bin/.cue files pair, so it can be burned and played in PSX! :-)
 */
public class Iso2Raw {
   private byte sec[] = new byte[16];
   private byte sub[] = new byte[8];
   private byte buf[] = new byte[2048];
   private byte edc[] = new byte[4];
   private byte ecc[] = new byte[276];
   
   /** The class' constructos. Just initializes some important arrays! */   
   public Iso2Raw() {
      memset(sec, (byte)0xFF);
      sec[0]  = 0;
      sec[11] = 0;
      sec[12] = 0;
      sec[13] = 2;
      sec[14] = 0;
      sec[15] = 2;
      
      memset(sub, (byte)0x00);
      memset(edc, (byte)0x01);
      memset(ecc, (byte)0x02);
   }
   
   private void memset(byte array[], byte value) {
      for (int i = 0; i < array.length; i++)
         array[i] = value;
   }
   
   private boolean licenseFile(String licFile, String binFile) {
      RandomAccessFile lic, bin;
      byte buffer[] = new byte[37632];
      
      try {
         lic = new RandomAccessFile(licFile, "r");
         bin = new RandomAccessFile(binFile, "rw");
         
         if (lic.length() != 37632) {
            System.out.println("Error! License file size mismatch. Image not licensed!");
            return false;
         }
         
         if ((bin.length() % 2352) != 0) {
            System.out.println("Error! RAW image file size is not a multiple of 2352. Image not licensed!");
            return false;
         }
         
         lic.read(buffer);
         bin.write(buffer);
         
         lic.close();
         bin.close();
      } catch (Exception e) {
         System.out.println("Error licensing file! You must NOT burn the RAW image!");
         return false;
      }
      
      return true;
   }
   
   private void generateCue(String binFileName) {
      File f = new File(binFileName);
      
      try {
         RandomAccessFile cue = new RandomAccessFile(f.getAbsoluteFile().toString().replaceAll("\\..*",".cue"), "rw");;
         cue.setLength(0);
         cue.writeBytes("FILE \"" + f.getAbsoluteFile().toString() + "\" BINARY\n");
         cue.writeBytes("  TRACK 01 MODE2/2352\n");
         cue.writeBytes("    INDEX 01 00:00:00\n");
         cue.close();
      } catch (Exception e) {
         return;
      }
   }
   
   /** This is the method which really do the job. It requires 3 parameters: iso file
    * name, bin file name (to be generated) and the license file (used to build PSX
    * CD-ROMs).
    * @param isofile Points to an existant file in the disk which is in plain ISO format.
    * @param rawfile Points to the file that will be generated.
    * @param licfile Points to an existant and valid license file.
    * @return This method return true on succes or false on fail.
    */   
   public boolean convert(String isofile, String rawfile, String licfile) {
      RandomAccessFile infile, outfile;
      int c;
      int thesec = 0;
      long filesize, totalsectors, sector;
     
      try {
         infile = new RandomAccessFile(isofile, "r");
         filesize = infile.length();
      } catch (Exception e) {
         System.err.println("An error has occurred while trying to open file " + isofile);
         return false;
      }
      
      if ((filesize % 2048) != 0) {
         System.err.println("Error! ISO file size is not a multiple of 2048. Operation aborted!");
         return false;
      }
      
      try {
         outfile = new RandomAccessFile(rawfile, "rw");
         outfile.setLength(0);
      } catch (Exception e) {
         System.err.println("An error has occurred while trying to create file " + rawfile);
         return false;
      }
      
      try {
         sector = 1;
         totalsectors = filesize / 2048;
         for(;;) {
            c = infile.read(buf);
            if (c == -1) break;
            
            outfile.write(sec);
            outfile.write(sub);
            outfile.write(buf);
            outfile.write(edc);
            outfile.write(ecc);
            
            thesec++;
            
            if (thesec > 74) {
               thesec = 0;
               sec[13]++;
            }
            
            sec[14] = (byte)Integer.parseInt(Integer.toString(thesec), 16);
            
            System.out.print("\r" + sector * 100 / totalsectors + "% completed...");
            sector++;
         }
         
         System.out.println("\r100% completed!  ");
   
         infile.close();
         outfile.close();

      } catch (Exception e) {
         System.err.println("An error has occurred while generating RAW file");
         return false;
      }
      
      generateCue(rawfile);
      
      if(!licenseFile(licfile, rawfile))
         return false;
      
      return true;
   }
}
